"""

 THE USER MUST EDIT THIS FILE TO SET THE BOUNDARY CONDITIONS

"""
import numpy
import scipy, scipy.interpolate
import anuga
import anuga.utilities.spatialInputUtil as su


def setup_boundary_conditions(domain,project):
    """
        THE USER MUST MODIFY THIS TO SETUP THE BOUNDARIES AS DESIRED

        1) First make sure the required boundary conditions are created
           (see examples below, modify or add new ones as required)

        2) Next make sure the 'boundary_tags_and_conditions' dictionary
           associates the correct 'tags' (contained as attribute names in the
           bounding_polygon_and_tags_file) with the correct boundary condition type
    """

#@@@@@@@@@@@@@@@@@#
#START EDITING HERE
#@@@@@@@@@@@@@@@@@#

    # Example:
    # "Standard" boundary conditions
    Br = anuga.Reflective_boundary(domain)
    #Bs = anuga.Transmissive_stage_zero_momentum_boundary(domain) # Be careful with this one
   
    # Example: 
    # Transmissive with stage being set by a function 'stage_function'
    def stage_function(t):
        return min( max( t-60., 0.)/60., 1.)
    B_t_stage_function = anuga.shallow_water.boundaries.Transmissive_n_momentum_zero_t_momentum_set_stage_boundary(domain, stage_function)

    ## Here we associate each tag in boundary_info with its boundary condition defined above
    boundary_tags_and_conditions={ 
                                  'Ocean': B_t_stage_function,
                                  'Br':Br 
                                  }

#@@@@@@@@@@@@@@@@@#
#STOP EDITING HERE
#@@@@@@@@@@@@@@@@@#

    ## Check that all tags in boundary_info have been set
    for i in range(len(project.boundary_tags)):
        tag=project.boundary_tags.keys()[i]
        if not boundary_tags_and_conditions.has_key(tag):
            msg = 'Need to set boundary_tags_and_conditions for tag = ' + tag
            raise Exception, msg
        

    domain.set_boundary(boundary_tags_and_conditions)

    return 

