"""

THIS IS NOT DESIGNED TO BE CHANGED BY THE USER 

Controlling variables are defined in 

project.py (polygons, resolutions etc)
setup_boundary_conditions.py (Data used to make boundary tags)


"""


import glob
import os
from os.path import join
import gc

import anuga
from anuga_parallel import distribute, myid, numprocs, finalize, barrier
from anuga.utilities import spatialInputUtil as su
from anuga_parallel.parallel_api import pypar_available
if pypar_available:
    from anuga import sequential_distribute_load
    from anuga import sequential_distribute_dump
    from anuga_parallel.sequential_distribute import sequential_distribute_load_pickle_file


#@@@@@@@@@@@@@@@@@#
#STOP EDITING HERE
#@@@@@@@@@@@@@@@@@#
#
# Yes, I mean stop - in theory it's not required.
# 


verbose=True

#####################################################################################

#def make_boundary_tags(bounding_polygon, setup_boundary_conditions):
#    """
#        Make the boundary tags 
#    """
#
#    # Close the bounding polygon
#    bounding_polygon_closed=bounding_polygon+[bounding_polygon[0]]
#
#    # Assign all tags to 'Default', but remove them from this list during
#    # assignment
#    default_tags=range(len(bounding_polygon))
#
#    # Dict for boundary tags
#    boundary_tags={}
#
#    # Information on the tags
#    boundary_info=setup_boundary_conditions.boundary_info
#
#    for i in range(len(boundary_info)):
#        bi=boundary_info[i]
#        # Shorthand for i'th boundary tag
#        tag_i=bi[0]
#        # Shorthand for i'th set of boundary points
#        PTS_i=bi[1]
#
#        # Add the boundary tag to the list
#        boundary_tags[tag_i]=[]
#
#        if(PTS_i=='Default' or i==(len(boundary_info)-1)):
#            # Assign default boundaries
#            assert (i==(len(boundary_info)-1) and PTS_i=='Default'),\
#                'Default boundary condition must be at the end of the boundary_info list'
#            boundary_tags[tag_i]=default_tags
#        else:
#            # Loop over all points in the boundary condition
#            for j in range(len(PTS_i)):
#                new_index=su.find_nearest_segment(PTS_i[j], bounding_polygon_closed)[1]
#                boundary_tags[tag_i]=boundary_tags[tag_i]+[new_index]     
#                default_tags.remove(new_index)
#
#    return boundary_tags    

#################################################################################################

def build_mesh(project, setup_boundary_conditions):
    """

        This is executed by processor 0 to build the mesh.

    """

    # Create the boundary tags
    #boundary_tags=make_boundary_tags(project.bounding_polygon, setup_boundary_conditions)
    
    # Ensure mesh_breaklines include riverwalls and breaklines
    mesh_breaklines=su.combine_breakLines_and_riverWalls_for_mesh(project.breaklines, project.riverwalls)

    ## Make the mesh
    anuga.create_mesh_from_regions(project.bounding_polygon,
                             boundary_tags=project.boundary_tags,
                             maximum_triangle_area=project.default_res,
                             filename=project.meshname,
                             interior_regions=project.interior_regions,
                             use_cache=False,
                             verbose=verbose,
                             breaklines=mesh_breaklines,
                             regionPtArea=project.region_point_areas)
                             
    # Make the domain using the mesh
    domain=anuga.create_domain_from_file(project.meshname)
    
    # Print some stats about mesh and domain
    print 'Number of triangles = ', len(domain)
    print 'The extent is ', domain.get_extent()
    print domain.statistics()
   
    # Print info on the smallest triangles
    smallAreas=domain.areas.argsort()
    print ''
    print 'LOCATIONS OF TRIANGLES WITH SMALLEST AREAS'
    for i in range(10):
        j=smallAreas[i]
        x = domain.centroid_coordinates[j,0]+domain.geo_reference.xllcorner
        y = domain.centroid_coordinates[j,1]+domain.geo_reference.yllcorner
        print '  Area ' + str(domain.areas[j]) + ' location: ' + str(round(x,1)) + ',' + str(round(y,1))
    print ''

    return(domain)

#####################################################################################

def setup_mesh(project, setup_boundary_conditions):
    """
    Code to make the mesh (initial domain)

    The geometry is made on processor 0, then dumped and reloaded
    This reduces the memory demands 

    INPUT: project == the project module

    OUTPUT: domain
    """
    
    if(myid==0):
        # Make sure files/folders to store everything are there
        project.setup_directory_structure()

        # Copy code files to output dir. This should be 'nearly the first' thing we do
        model_files=glob.glob('*.py')+glob.glob('*.sh')
        for model_file in model_files:
            anuga.copy_code_files(project.output_dir,model_file)
    

        if verbose: print 'Hello from processor ', myid

        #
        # HERE, WE MAKE/PARTITION/READ THE MESH
        # This can lead to memory savings in parallel runs
        # (possibly because of limitations of python memory management)
        # 
        
        # Let's see if we have already pickled this domain
        pickle_name = 'domain'+'_P%g_%g.pickle'% (1,0)
        pickle_name = join(project.partition_dir, pickle_name)

        if os.path.exists(pickle_name):
            if verbose: print 'Saved domain seems to already exist'
        else:
            if verbose: print 'CREATING PARTITIONED DOMAIN'
            domain=build_mesh(project, setup_boundary_conditions)
            # If pypar is not available don't use sequential_distribute stuff (it will fail)
            if pypar_available : 
                if verbose: print 'Saving Domain'
                sequential_distribute_dump(domain, 1, partition_dir=project.partition_dir, verbose=verbose)    

        # If pypar is not available don't use sequential_distribute stuff (it will fail)
        if pypar_available: 
            # Now partition the domain
            par_pickle_name = 'domain'+'_P%g_%g.pickle'% (numprocs,0)
            par_pickle_name = join(project.partition_dir,par_pickle_name)
            if os.path.exists(par_pickle_name):
                if verbose: print 'Saved partitioned domain seems to already exist'
            else:
                if verbose: print 'Load in saved sequential pickled domain'
                domain = sequential_distribute_load_pickle_file(pickle_name, np=1, verbose = verbose)
         
                if verbose: print 'Dump partitioned domains'
                sequential_distribute_dump(domain, numprocs, partition_dir=project.partition_dir, verbose=verbose) 
           
            # This can reduce the memory demands if the domain was made above
            # Even better to have an existing partition (i.e. stop here and rerun)
            domain=None     
            gc.collect() 

    else:
        domain=None
        if verbose: print 'Hello from processor ', myid

    barrier()

    #print 'Distributing domain'
    #domain=distribute(domain)
    #barrier()

    # If pypar is not available don't use sequential_distribute stuff (it will fail)
    if pypar_available:
        if myid == 0 : print 'LOADING PARTITIONED DOMAIN'
        
        domain = sequential_distribute_load(filename=join(project.partition_dir,'domain'), verbose = verbose)
    
    ###############################################################################
    # Set output directories 
    ###############################################################################
    domain.set_name(project.scenario) # Name of sww file
    domain.set_datadir(project.output_dir) # Store sww output here
   
    # Needs more changes for this to work 
    #domain.set_checkpointing(checkpoint_time=project.checkpoint_time)

    ###############################################################################
    # Miscellanious numerics
    ###############################################################################
    domain.set_flow_algorithm(project.flow_algorithm)

    # Force zero beta values [hopefully integrated into source]
    #print 'Warning: Forcing everything to first order'
    #domain.beta_w=0.
    #domain.beta_uh=0.
    #domain.beta_vh=0.
    #domain.beta_w_dry=0.
    #domain.beta_uh_dry=0.
    #domain.beta_vh_dry=0.

    # Adjust velocity computation for max quantities
    #domain.velocity_protection=1.0e-05

    # Adjust CFL
    #domain.set_CFL(0.9)

    # Optionally store vertex values uniquely (large file sizes!)
    domain.set_store_vertices_uniquely(project.store_vertices_uniquely)

    if project.use_local_extrapolation_and_flux_updating:
        domain.set_local_extrapolation_and_flux_updating()

    return(domain)

